"""Provide a command-line interface to use cardinal_words and ordinal_words."""

from argparse import ArgumentParser

from num2fawords import words, ordinal_words


parser = ArgumentParser()
parser.add_argument(
    'number', help='the number that is going to be converted to words'
)
parser.add_argument(
    '--ordinal', '-o',
    help='convert to ordinal from', action='store_true',
)
args = parser.parse_args()
if args.ordinal:
    print(ordinal_words(args.number))
else:
    print(words(args.number))



from multiprocessing import Pool
from functools import partial


def par(files , func , worker=4 , args=[]):
  with Pool(processes=worker) as pool:
     out_res = pool.map(partial(func, _args=args), files)
  return  out_res 

  



from queue import Queue
import threading

q= Queue()


def worker():
    while True:  
         item = q.get()
        
         #train
        
         q.task_done()
         
threading.Thread(target=worker, daemon=True).start()




while(True):
     while (q.qsize()>5):
         time.sleep(1)  
         
     #create data
     
     q.put(data)
 
            
q.join()   


import requests

S = requests.Session()

URL = "https://fa.wikipedia.org/w/api.php"

SEARCHPAGE = "روحانی"

PARAMS = {
    "action": "query",
    "format": "json",
    "list": "search",
    "srsearch": SEARCHPAGE
}

R = S.get(url=URL, params=PARAMS)
DATA = R.json()

for x in DATA['query']['search']:
    if SEARCHPAGE.lower() in x['title']:
      print(x['title'])
      print(x['snippet'])
      print('***************')




# -*- coding: utf-8 -*-
"""
Created on Tue Apr  9 12:07:27 2019

@author: deepspeech
"""

import librosa as l
import numpy as np
import soundfile as sf

wav, sr = l.load('test.wav', sr=16000)
st = l.stft(y=wav , n_fft=256 , hop_length=128 , win_length=256)
mag = np.abs(st)
phase  = np.angle(st) 

stft = mag * np.exp(1.j * phase)

wav2 = l.istft(stft, win_length=256, hop_length=128)

sf.write('test2.wav', wav2, 16000, format='wav', subtype='PCM_16')







#pip install graphviz

import graphviz

dot = graphviz.Digraph('round-table', comment='The Round Table') 

dot.node('A', 'Speech & Text Processing', color='red')  
dot.node('S', 'Speech Processing', color='blue')

dot.edge('A','S', color='blue' )

dot.render(directory='doctest-output', view=True)


import tensorflow as tf
from tensorflow.compat.v1.keras.backend import set_session
config = tf.compat.v1.ConfigProto()
config.gpu_options.allow_growth = True 
config.log_device_placement = True 
sess = tf.compat.v1.Session(config=config)
set_session(sess) 


# python-docx==0.8.11
# lxml


from docx import Document
from docx.shared import Inches
from docx.enum.text import WD_ALIGN_PARAGRAPH 


document = Document()

document.add_heading('Document Title', 0)

p = document.add_paragraph('A plain paragraph having some ')
p.add_run('bold').bold = True
p.add_run(' and some ')
p.add_run('italic.').italic = True
p.alignment=WD_ALIGN_PARAGRAPH.CENTER  # for left, 1 for center, 2 right, 3 justify ....

document.add_heading('Heading, level 1', level=1)
document.add_paragraph('Intense quote', style='Intense Quote')

document.add_paragraph(
    'first item in unordered list', style='List Bullet'
)
document.add_paragraph(
    'first item in ordered list', style='List Number'
)

# document.add_picture('monty-truth.png', width=Inches(1.25))

records = (
    (3, '101', 'Spam'),
    (7, '422', 'Eggs'),
    (4, '631', 'Spam, spam, eggs, and spam')
)

table = document.add_table(rows=1, cols=3)
hdr_cells = table.rows[0].cells
hdr_cells[0].text = 'Qty'
hdr_cells[1].text = 'Id'
hdr_cells[2].text = 'Desc'
for qty, id, desc in records:
    row_cells = table.add_row().cells
    row_cells[0].text = str(qty)
    row_cells[1].text = id
    row_cells[2].text = desc

document.add_page_break()

document.save('demo.docx')



doc = docx.Document()
style = doc.styles.add_style('rtl', WD_STYLE_TYPE.PARAGRAPH)
# style.font.rtl = True
style.font.name = 'Nazanin'
style.font.size = Pt(14)

p = doc.add_paragraph(style='rtl')
p.alignment=WD_ALIGN_PARAGRAPH.RIGHT
p.add_run('بنام خدا')

doc.save('iman.docx')


from iman import *

x= cmd('conda info')
idx = x.index('active env location :')
idy=x.index('shell level' , idx)
p = PJ(x[idx+22:idy].strip() ,'Lib','site-packages','iman','examples_folder')

help_items = gf(PJ(p,'*.py'))

def read_lines(_path):
  lines=[]
  with open (_path ,'r' ,encoding='utf-8') as fid:
     for line in fid:
        lines.append(line.strip('\n'))
  return(lines)      
 
items =  [PN(x) for x in help_items]

def help(_str):
    for x in help_items:
          if (_str in PN(x)):
             print('')
             [print( y ) for y in read_lines(x)]
             print('')
             return
    return('Nothing Found')         






"""Provide functions to convert a number to Persian words."""

from decimal import Decimal
from fractions import Fraction
try:
    from functools import singledispatch as _singledispatch
except ImportError:  # pragma: no cover
    # sys.version_info < (3, 4)
    # noinspection PyUnresolvedReferences
    from singledispatch import singledispatch as _singledispatch
from itertools import chain as _chain
from typing import Union as _Union


ONES = [
    '',
    'یک',
    'دو',
    'سه',
    'چهار',
    'پنج',
    'شش',
    'هفت',
    'هشت',
    'نه',
]

TENS = [
    '',
    '',
    'بیست',
    'سی',
    'چهل',
    'پنجاه',
    'شصت',
    'هفتاد',
    'هشتاد',
    'نود',
]

TEN_TO_TWENTY = [
    'ده',
    'یازده',
    'دوازده',
    'سیزده',
    'چهارده',
    'پانزده',
    'شانزده',
    'هفده',
    'هجده',
    'نوزده',
]

HUNDREDS = [
    '',
    'یکصد',
    'دویست',
    'سیصد',
    'چهارصد',
    'پانصد',
    'ششصد',
    'هفتصد',
    'هشتصد',
    'نهصد',
]

CLASSES = [
    '',
    ' هزار',
    ' میلیون',
    ' میلیارد',
    ' بیلیون',
    ' بیلیارد',
    ' تریلیون',
    ' ترلیارد',
    ' کوآدریلیون',
    ' کادریلیارد',
    ' کوینتیلیون',
    ' کوانتینیارد',
]

DECIMAL_PLACES = ['', ' دهم', ' صدم']
DECIMAL_PLACES.extend(_chain.from_iterable(
    (i, ' ده' + i, ' صد' + i)
    for i in (i + 'م' for i in CLASSES[1:])
))

_NORMALIZATION_TABLE = str.maketrans('E٫', 'e.', '_٬,+')


def _three_digit_words(number: int) -> str:
    """Return the word representation of 0 < number < 1000."""
    h, t, o = number // 100, number % 100 // 10, number % 10
    if h == 0 or (t == o == 0):
        w = HUNDREDS[h]
    else:
        w = HUNDREDS[h] + ' و '
    if t == 1:
        return w + TEN_TO_TWENTY[o]
    if o == 0 or t == 0:
        w += TENS[t]
    else:
        w += TENS[t] + ' و '
    return w + ONES[o]


# noinspection PyUnusedLocal
@_singledispatch
def words(
    number: _Union[int, float, str, Decimal, Fraction],
    positive: str='',
    negative: str='منفی ',
    decimal_separator: str=' و ',
    fraction_separator: str=' ',
    ordinal_denominator: bool=True,
    scientific_separator: str=' در ده به توان ',
) -> str:
    """Return the word form of number.

    If input is a string it should be in the form of a valid Python
    representation for one of the other accepted types. The only exceptions are
    that digits can be in Persian, for example words('۴۲') is valid.

    """
    raise TypeError('invalid input type for words function', number)


@words.register(str)
@words.register(Decimal)
def _(
    number: str,
    positive: str='',
    negative: str='منفی ',
    decimal_separator: str=' و ',
    fraction_separator: str=' ',
    ordinal_denominator: bool=True,
    scientific_separator: str=' در ده به توان ',
) -> str:
    # Normalize the str
    number = str(number).strip().translate(_NORMALIZATION_TABLE)

    # sign
    c0 = number[0]
    if c0 == '-':
        sign = negative
        number = number[1:]
    elif c0 == '0':
        sign = ''
    else:
        sign = positive

    numerator, e, denominator = number.partition('/')

    if denominator:
        if ordinal_denominator:
            return (
                sign
                + _natural_words(numerator)
                + fraction_separator
                + ordinal_words(denominator)
            )
        return (
            sign
            + _natural_words(numerator)
            + fraction_separator
            + _natural_words(denominator)
        )
    return sign + _exp_words(
        numerator, positive, negative, decimal_separator, scientific_separator,
    )


# noinspection PyUnusedLocal
@words.register(Fraction)
def _(
    number: Fraction,
    positive: str='',
    negative: str='منفی ',
    decimal_separator: str=' و ',
    fraction_separator: str=' ',
    ordinal_denominator: bool=True,
    scientific_separator: str=' در ده به توان ',
) -> str:
    numerator = number.numerator
    if numerator < 0:
        sign = negative
        numerator = str(numerator)[1:]
    else:
        sign = positive
        numerator = str(numerator)
    if ordinal_denominator:
        return (
            sign
            + _natural_words(numerator)
            + fraction_separator
            + ordinal_words(number.denominator)  # denominator has no sign
        )
    return (
        sign
        + _natural_words(numerator)
        + fraction_separator
        + _natural_words(str(number.denominator))  # denominator has no sign
    )


# noinspection PyUnusedLocal
@words.register(int)
def _(
    number: int,
    positive: str='',
    negative: str='منفی ',
    decimal_separator: str=' و ',
    fraction_separator: str=' ',
    ordinal_denominator: bool=True,
    scientific_separator: str=' در ده به توان ',
) -> str:
    """Return the fa-word form for the given int."""
    if number == 0:
        return 'صفر'
    if number < 0:
        return negative + _natural_words(str(number)[1:])
    return positive + _natural_words(str(number))


# noinspection PyUnusedLocal
@words.register(float)
def _(
    number: float,
    positive: str='',
    negative: str='منفی ',
    decimal_separator: str=' و ',
    fraction_separator: str=' ',
    ordinal_denominator: bool=True,
    scientific_separator: str=' در ده به توان ',
) -> str:
    """Return the fa-word form for the given float."""
    if number == 0:
        return 'صفر'
    str_num = str(number)
    if number < 0:
        return negative + _exp_words(
            str_num[1:],
            positive,
            negative,
            decimal_separator,
            scientific_separator,
        )
    return positive + _exp_words(
        str_num, positive, negative, decimal_separator, scientific_separator,
    )


def _exp_words(
    number: str,
    positive: str,
    negative: str,
    decimal_separator: str,
    scientific_separator: str,
) -> str:
    # exponent
    base, e, exponent = number.partition('e')
    if exponent:
        return (
            _point_words(base, decimal_separator=decimal_separator)
            + scientific_separator
            + words(int(exponent), positive, negative)
        )
    return _point_words(base, decimal_separator=decimal_separator)


def _point_words(
    number: str,
    decimal_separator: str,
) -> str:
    before_p, p, after_p = number.partition('.')
    if after_p:
        if before_p == '0':
            if after_p == '0':
                return 'صفر'
            return _natural_words(after_p) + DECIMAL_PLACES[len(after_p)]
        if after_p != '0':
            return (
                _natural_words(before_p)
                + decimal_separator
                + _natural_words(after_p)
                + DECIMAL_PLACES[len(after_p)]
            )
        return _natural_words(before_p)
    return _natural_words(before_p)


def _natural_words(str_num: str) -> str:
    if str_num == '0':
        return 'صفر'
    length = len(str_num)
    if length > len(CLASSES) * 3:
        raise ValueError('out of range')

    modulo_3 = length % 3
    if modulo_3:
        str_num = '0' * (3 - modulo_3) + str_num
        length += 3 - modulo_3

    groups = length // 3
    group = groups
    natural_words = ''
    while group > 0:
        three_digit = str_num[group * 3 - 3:group * 3]
        word3 = _three_digit_words(int(three_digit))
        if word3 and group != groups:
            if natural_words:
                natural_words = word3 + CLASSES[groups - group] + \
                                ' و ' + natural_words
            else:
                natural_words = word3 + CLASSES[groups - group]
        else:
            natural_words = word3 + natural_words
        group -= 1

    return natural_words


def ordinal_words(
    number: _Union[int, str],
    positive: str='',
    negative: str='منفی ',
)-> str:
    """Return the number converted to ordinal words form."""
    w = words(int(number), positive, negative)
    if w[-2:] == 'سه':
        return w[:-2] + 'سوم'
    return w + 'م'


def change_defaults(
    positive: str='',
    negative: str='منفی ',
    decimal_separator: str=' و ',
    fraction_separator: str=' ',
    ordinal_denominator: bool=True,
    scientific_separator: str=' در ده به توان ',
):
    """The the default values for words and ordinal_words functions."""
    defaults = (
        positive, negative, decimal_separator, fraction_separator,
        ordinal_denominator, scientific_separator,
    )
    change_defaults.__defaults__ = defaults
    for func in words.registry.values():
        func.__defaults__ = defaults
    ordinal_words.__defaults__ = (positive, negative)



from torch.utils.tensorboard import SummaryWriter


class rep():
    def __init__(self,log_dir=None):
      self.log_dir=log_dir  
      self.writer = SummaryWriter(log_dir=log_dir)      


    def WS(self,_type , _name , value , itr):
       self.writer.add_scalar(_type + '/' + _name, value, itr)
       
    def WT(self,_type , _name , _str , itr):  
        self.writer.add_text(_type + '/' + _name, _str, itr)
        
    def WG(self,pytorch_model , example_input):
       self.writer.add_graph(pytorch_model,example_input)
       
    def WI(self,_type , _name , images , itr):
        import torchvision
        grid = torchvision.utils.make_grid(images)
        self.writer.add_image(_type + '/' + _name, grid, itr) 
        
    def WC(self):
       self.writer.close()    
      